from trac.core import *
from ldapplugin.api import *
import ldap
import acct_mgr.api

class LdapAuthStore(Component):
    """A trac authentication provider using LDAP.
    
    The LDAP database is configured with LdapPlugin.
    Registers itself with the AuthManager trac plugin.
    """
    implements(acct_mgr.api.IPasswordStore)

    def __init__(self, ldap=None):
        # looks for groups only if LDAP support is enabled
        self.enabled = self.config.getbool('ldap', 'enable')
        if not self.enabled:
            return
        
        self.util = LdapUtil(self.config)
        # LDAP connection
        self._ldap = ldap
        # LDAP connection config
        self._ldapcfg = {}
        for name,value in self.config.options('ldap'):
            if name in LDAP_DIRECTORY_PARAMS:
                self._ldapcfg[name] = value
        # group of all users with permission to log in
        self._allusers = self.config.get('ldap',
                'allusers_group', 'tracusers')

    def has_user(self, user):
        """Returns whether the user account exists.
        
        Does so by checking if the user is a member of
        the ldap.allusers LDAP group."""

        dn = self.util.user_attrdn(user)
        gdn = self.util.group_attrdn(self._allusers)

        self._openldap()
        return self._ldap.is_in_group(dn, gdn)

    def get_users(self):
        """Returns an iterable of the known usernames.
        
        Does so by listing the members of the LDAP group ldap.allusers .
        """

        self._openldap()
        gdn = self.util.group_attrdn(self._allusers)
        ldap_users = self._ldap.get_attribute(gdn,
                self.config.get('ldap', 'groupmember', 'member')
                .encode('ascii'))
        self.env.log.debug('Found %d trac users in LDAP directory.' \
        % len(ldap_users))
        if not self.config.getbool('ldap', 'groupmemberisdn', True):
                return ldap_users
        return (self.util.extract_user_from_dn(dn) for dn in ldap_users)


    def set_password(self, user, password, old_password = None):
        dn = self.util.user_attrdn(user)
        try:
            self._set_password_as_user(dn, password, old_password)
        except ldap.UNWILLING_TO_PERFORM:
            self._set_password_as_admin(user, password, old_password)
        #Means no creation - weird convention
        #This means we send exceptions for anything else
        return False

    def _set_password_as_user(self, dn, password, old_password):
        """Throws if it cannot."""
        cnx = self._bind_as_user(dn, old_password)
        try:
            cnx._ds.passwd_s(dn, old_password, password)
        except ldap.UNWILLING_TO_PERFORM:
            cnx.close()
            raise

    def _set_password_as_admin(self, dn, password, old_password):
        self._openldap()
        self._ldap._open()
        self._ldap._ds.passwd_s(dn, old_password, password)

    def check_password(self, user, password):
        """Checks if the password is valid for the user.
        
        Does so by attempting an LDAP bind as this user.
        Does not attempt to check this is a valid trac account,
        see has_user for that.
        """
        dn = self.util.user_attrdn(user)
        cnx = self._bind_as_user(dn, password)
        if cnx is None:
            return False
        cnx.close()
        return True
    
    def _bind_as_user(self, dn, password):
        """Returns an ldapplugin.LdapConnection for this user.
        
        Returns None if unsuccessful.
        You must close the connection by calling its close method.
        """

        #   Creating an LdapConnection is maybe expensive, but sharing
        # _ldap would not be safe.
        #   The integration with LdapPlugin could
        # be better: add an API to LdapPlugin that gives us a raw,
        # unbound connection to the server.
        settings = dict(self._ldapcfg)
        settings['bind_user'] = dn
        settings['bind_passwd'] = password
        try:
            lc = LdapConnection(self.env.log, True, **settings)
            lc._open()
            return lc
        except TracError:
            try:
                lc.close()
            except:
                pass
        return None

    def delete_user(self, user):
        """Deletes the user account.

        Returns True if the account existed and was deleted, False otherwise.
        """
        return False

    def _openldap(self):
        """Ensure self._ldap is set to a privileged LDAP connection.
        """
        if self._ldap is None: 
            bind = self.config.getbool('ldap', 'store_bind') or \
                self.config.getbool('ldap', 'group_bind')
            self._ldap = LdapConnection(self.env.log, bind, **self._ldapcfg)

# vim: set sw=4 ts=4 et :
